"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Source: https://github.com/madrobby/keymaster/blob/3b1f2afabf1569848dea8b697ac418f19b601a30/keymaster.js
 */
exports.keyMap = {
    '⇧': 16, shift: 16,
    '⌥': 18, alt: 18, option: 18,
    '⌃': 17, ctrl: 17, control: 17,
    '⌘': 91, cmd: 91, command: 91, meta: 91,
    caps: 20, capslock: 20,
    backspace: 8, tab: 9, clear: 12,
    enter: 13, return: 13,
    esc: 27, escape: 27, space: 32,
    left: 37, up: 38,
    right: 39, down: 40,
    del: 46, delete: 46,
    home: 36, end: 35,
    pageup: 33, pagedown: 34,
    ',': 188, '.': 190, '/': 191,
    '`': 192, '-': 189, '=': 187,
    ';': 186, '\'': 222,
    '[': 219, ']': 221, '\\': 220
};
/**
 * Valid modifier keys.
 */
exports.modifiers = [16, 17, 18, 91, 93, 224];
/**
 * Stringify a keyboard event.
 */
function stringifyKey(e) {
    var keys = [];
    if (e.shiftKey)
        keys.push(exports.keyMap.shift);
    if (e.ctrlKey)
        keys.push(exports.keyMap.ctrl);
    if (e.altKey)
        keys.push(exports.keyMap.alt);
    if (e.metaKey)
        keys.push(exports.keyMap.meta);
    if (exports.modifiers.indexOf(e.which) === -1)
        keys.push(e.which);
    return keys.sort().join(' ');
}
exports.stringifyKey = stringifyKey;
/**
 * Keyboard manager library for mapping key events.
 */
var Keyboard = /** @class */ (function () {
    function Keyboard(map) {
        if (map === void 0) { map = exports.keyMap; }
        var _this = this;
        this.map = map;
        this.listeners = [];
        this.shortcuts = Object.create(null);
        this.onKeyDown = function (e) {
            var combo = stringifyKey(e);
            for (var _i = 0, _a = _this.listeners; _i < _a.length; _i++) {
                var callback = _a[_i];
                callback(combo, e);
            }
        };
        // Listen and map to shortcut handler.
        this.listen(function (combo, e) {
            var callbacks = _this.shortcuts[combo];
            if (!callbacks)
                return;
            var callback = callbacks[callbacks.length - 1];
            callback(e);
        });
    }
    Keyboard.prototype.mapKeys = function (keys) {
        var _this = this;
        if (!Array.isArray(keys))
            return keys;
        return keys.map(function (key) {
            if (typeof key === 'number')
                return key;
            if (_this.map[key])
                return _this.map[key];
            if (key.length > 1)
                throw new TypeError("Unknown key: " + key);
            return key.toUpperCase().charCodeAt(0);
        }).sort().join(' ');
    };
    Keyboard.prototype.listen = function (callback) {
        return this.listeners.push(callback);
    };
    Keyboard.prototype.stopListening = function (callback) {
        var indexOf = this.listeners.indexOf(callback);
        if (indexOf > -1)
            this.listeners.splice(indexOf, 1);
        return indexOf;
    };
    Keyboard.prototype.bind = function (keys, callback) {
        var key = this.mapKeys(keys);
        if (!this.shortcuts[key])
            this.shortcuts[key] = [];
        return this.shortcuts[key].push(callback);
    };
    Keyboard.prototype.unbind = function (keys, callback) {
        var key = this.mapKeys(keys);
        if (!this.shortcuts[key])
            return -1;
        var indexOf = this.shortcuts[key].indexOf(callback);
        if (indexOf > -1)
            this.shortcuts[key].splice(indexOf);
        if (!this.shortcuts[key].length)
            delete this.shortcuts[key];
        return indexOf;
    };
    Keyboard.prototype.attach = function (element) {
        if (element === void 0) { element = window; }
        element.addEventListener('keydown', this.onKeyDown, false);
    };
    Keyboard.prototype.detach = function (element) {
        if (element === void 0) { element = window; }
        element.removeEventListener('keydown', this.onKeyDown, false);
    };
    return Keyboard;
}());
exports.Keyboard = Keyboard;
//# sourceMappingURL=index.js.map